var p6500 = null;
var s1hum;
var s1Duty;
var tankPres;
var outletPres;
var s2hum;
var flow;
var s2Duty;
var compTemp;
var cabTemp;
var simulateCompressorEnable = false;
var blinkColor = "#df7354"
var socket;



Number.prototype.pad = function(size) {
	var s = String(this);
	while (s.length < (size || 2)) {s = "0" + s;}
	return s;
}

/**
 * Open a new WebSocket connection using the given parameters
 */
function createGages()
{
	s1hum = new JustGage({
		id : "s1Humidity",
		value : 0,
		min : 0,
		max : 15,
		title : "Humidity",
		label : "%",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black",
		formatNumber: true,
		decimals : 1
	});
	s1Duty = new JustGage({
		id : "s1Duty",
		value : 50,
		min : 0,
		max : 100,
		title : "Duty Cycle",
		label : "%",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black"
	});
	tankPres = new JustGage({
		id : "tankPressure",
		value : 0,
		min : 0,
		max : 60,
		title : "Tank Pressure",
		label : "PSI",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black",
		formatNumber: true,
		decimals : 1
	});
	outletPres = new JustGage({
		id : "outletPressure",
		value : 0,
		min : 0,
		max : 30,
		title : "Outlet Pressure",
		label : "PSI",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black",
		formatNumber: true,
		decimals : 1
	});
	flow = new JustGage({
		id : "flow",
		value : 0,
		min : 0,
		max : 6500,
		title : "Flow",
		label : "SCFD",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black",
		formatNumber: true,
		decimals : 1
	});
	s2hum = new JustGage({
		id : "s2Humidity",
		value : getRandomInt(0, 100),
		min : 0,
		max : 15,
		title : "Humidity",
		label : "%",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black",
		formatNumber: true,
		decimals : 1
	});
	s2Duty = new JustGage({
		id : "s2Duty",
		value : 50,
		min : 0,
		max : 100,
		title : "Duty Cycle",
		label : "%",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black"
	});
	compTemp = new JustGage({
		id : "compTemp",
		value : getRandomInt(30, 200),
		min : 0,
		max : 200,
		title : "Compressor Temp",
		label : "Deg",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black",
		formatNumber: true,
		decimals : 1
	});
	cabTemp = new JustGage({
		id : "cabTemp",
		value : getRandomInt(30, 200),
		min : 0,
		max : 200,
		title : "Cabinet Temp",
		label : "Deg",
		titleFontColor : "white",
		titleFontFamily : "roboto-regular",
		valueFontColor : "white",
		valueFontFamily : "roboto-black",
		formatNumber: true,
		decimals : 1
	});}

function setupWebSocket(webSocket)
{
	try {
		
		webSocket.onopen = function(openEvent) {
			console.log("WebSocket OPEN: " + JSON.stringify(openEvent, null, 4));

		};
		webSocket.onclose = function (closeEvent) {
			console.log("WebSocket CLOSE: " + JSON.stringify(closeEvent, null, 4));
		};
		webSocket.onerror = function (errorEvent) {
			console.log("WebSocket ERROR: " + JSON.stringify(errorEvent, null, 4));
		};
		webSocket.onmessage = function (messageEvent) {
			var wsMsg = messageEvent.data;
			console.log("WebSocket MESSAGE: " + wsMsg);
			p6500 = JSON.parse(wsMsg);
			updateCycleMode();
			updateSystems();
			updatePressures();
			updateTemps();
			updateCompressorStatus();
			updateButtonState();
			updateDutyCycle();
		};
		socket = webSocket;
	} catch (exception) {
		console.error(exception);
	}
}

function runSimulation()
{
	if(simulateCompressorEnable)
	{
		simulateCompressor()
	}
}
function toggleCompressorSimulation()
{
	

	var bttn = document.getElementById("enableCompressorSim");
	if(simulateCompressorEnable)
	{
		bttn.innerHTML = "Simulate Compressor";
		bttn.style.color = "#e4c352";
		simulateCompressorEnable = false;

	}
	else
	{
		if(!p6500.emulationEnabled)
		{
			alert("Sensor Emulation Must Be Enabled")
			return;
		}
		bttn.innerHTML = "Disable Compressor Simulation";
		bttn.style.color = "#52d073";
		simulateCompressorEnable = true;
	}
}

function enableEmulation()
{
	var bttn = document.getElementById("enableEmulation");
	if(p6500.emulationEnabled)
	{
		bttn.innerHTML = "Enable Emulation";
		sendMessage(socket,"DISABLE EMULATION")
		bttn.style.color = "#e4c352";
		if(simulateCompressorEnable)
		{
			toggleCompressorSimulation();
		}

	}
	else
	{
		sendMessage(socket,"ENABLE EMULATION")
		bttn.innerHTML = "Disable Emulation";
		bttn.style.color = "#52d073";
	}
}
function updateButtonState()
{
	var bttn = document.getElementById("enableEmulation");
	if(p6500.emulationEnabled)
	{
		bttn.innerHTML = "Disable Emulation";
		bttn.style.color = "#52d073";

	}
	else
	{
		bttn.innerHTML = "Enable Emulation";
		bttn.style.color = "#e4c352";
	}

	var bttn = document.getElementById("enableCompressorSim");
	if(simulateCompressorEnable)
	{
		bttn.innerHTML = "Disable Compressor Simulation";
		bttn.style.color = "#52d073";

	}
	else
	{
		bttn.innerHTML = "Simulate Compressor";
		bttn.style.color = "#e4c352";
	}
}
function updateCompressorStatus()
{
	if(p6500.systems[0].compressorEnabled)
	{
		document.getElementById('motor1').src = "images/motor_on.svg";
		document.getElementById('comp1Stat').innerHTML = "Compressor Running";
	}
	else
	{
		document.getElementById('motor1').src = "images/motor_off.svg";
		document.getElementById('comp1Stat').innerHTML = "Compressor Off";
	}
	if(p6500.systems[1].compressorEnabled)
	{
		document.getElementById('motor2').src = "images/motor_on.svg";
		document.getElementById('comp2Stat').innerHTML = "Compressor Running";
	}
	else
	{
		document.getElementById('motor2').src = "images/motor_off.svg";
		document.getElementById('comp2Stat').innerHTML = "Compressor Off";
	}
}

function blink(element, blinkColor)
{
	element.style.backgroundColor = blinkColor;

}
function blinkAlarms()
{
	if(p6500.alarms.alarms[0] == true || p6500.alarms.alarms[1] == true)
	{
		//low/high outlet
		blink(document.getElementById('outletPressure'), blinkColor);
	}
	else
	{
		blink(document.getElementById('outletPressure'), "#444");
	}
	if(p6500.alarms.alarms[2] == true)
	{
		//high flow
		blink(document.getElementById('flow'), blinkColor);
	}
	else
	{
		blink(document.getElementById('flow'), "#444");
	}
	if(p6500.alarms.alarms[3] == true)
	{
		//high comp temp
		blink(document.getElementById('compTemp'), blinkColor);
	}
	else
	{
		blink(document.getElementById('compTemp'), "#444");
	}
	if(p6500.alarms.alarms[4] == true)
	{
		//high cab temp
		blink(document.getElementById('cabTemp'), blinkColor);
	}
	else
	{
		blink(document.getElementById('cabTemp'), "#444");
	}
	
	if(p6500.alarms.alarms[5] == true)
	{
		//high cab temp
		blink(document.getElementById('s1Humidity'), blinkColor);
	}
	else
	{
		blink(document.getElementById('s1Humidity'), "#444");
	}
	
	if(p6500.alarms.alarms[6] == true)
	{

		blink(document.getElementById('s2Humidity'), blinkColor);
	}
	else
	{
		blink(document.getElementById('s2Humidity'), "#444");
	}
	if(p6500.alarms.alarms[7] == true)
	{
		//high cab temp
		blink(document.getElementById('sysLastRuntime'), blinkColor);
	}
	else
	{
		blink(document.getElementById('sysLastRuntime'), "#444");
	}

	if(p6500.alarms.alarms[8] == true)
	{
		//high cab temp
		blink(document.getElementById('sys2LastRuntime'), blinkColor);
	}
	else
	{
		blink(document.getElementById('sys2LastRuntime'), "#444");
	}

	if(p6500.alarms.alarms[9] == true)
	{
		//high cab temp
		blink(document.getElementById('sys1TotalRuntime'), blinkColor);
	}
	else
	{
		blink(document.getElementById('sys1TotalRuntime'), "#444");
	}
	if(p6500.alarms.alarms[10] == true)
	{
		//high cab temp
		blink(document.getElementById('sys2TotalRuntime'), blinkColor);
	}
	else
	{
		blink(document.getElementById('sys2TotalRuntime'), "#444");
	}
	if(p6500.alarms.alarms[11] == true)
	{
		//high cab temp
		blink(document.getElementById('s1Duty'), blinkColor);
	}
	else
	{
		blink(document.getElementById('s1Duty'), "#444");
	}
	if(p6500.alarms.alarms[12] == true)
	{
		//high cab temp
		blink(document.getElementById('s2Duty'), blinkColor);
	}
	else
	{
		blink(document.getElementById('s2Duty'), "#444");
	}

	if(blinkColor == "#444")
	{
		blinkColor = "#df7354"
	}

	else
	{
		blinkColor = "#444"
	}



}
function onClearAlarms()
{
	sendMessage(socket,"CLEAR ALARMS")
	document.getElementById('outletPressure').style.backgroundColor = "#444"
	document.getElementById('flow').style.backgroundColor = "#444"
	document.getElementById('compTemp').style.backgroundColor = "#444"
	document.getElementById('cabTemp').style.backgroundColor = "#444"
	document.getElementById('s1Humidity').style.backgroundColor = "#444"
	document.getElementById('s2Humidity').style.backgroundColor = "#444"
	document.getElementById('sysLastRuntime').style.backgroundColor = "#444"
	document.getElementById('sys2LastRuntime').style.backgroundColor = "#444"
	document.getElementById('sys1TotalRuntime').style.backgroundColor = "#444"
	document.getElementById('sys2TotalRuntime').style.backgroundColor = "#444"
	document.getElementById('s1Duty').style.backgroundColor = "#444"
	document.getElementById('s2Duty').style.backgroundColor = "#444"

}
function updatePressures()
{

	if(p6500.sensors.metricUnitsEnabled)
	{
		tankPres.refresh(p6500.sensors.tankPressure.toFixed(2),414);
		tankPres.refreshLabel("KPA")
		outletPres.refresh(p6500.sensors.outletPressure.toFixed(2),207);
		outletPres.refreshLabel("KPA")
		flow.refresh(p6500.sensors.flow.toFixed(2),185);
		flow.refreshLabel("SCMD")
	}
	else{
		tankPres.refresh(p6500.sensors.tankPressure.toFixed(2),60);
		tankPres.refreshLabel("PSI")
		outletPres.refresh(p6500.sensors.outletPressure.toFixed(2),30);
		outletPres.refreshLabel("PSI")
		flow.refresh(p6500.sensors.flow.toFixed(2),6500);
		flow.refreshLabel("SCMD")
	}


	
}
function updateDutyCycle()
{
	s1Duty.refresh(p6500.system1.dutyCycle);
	s2Duty.refresh(p6500.system2.dutyCycle);
}
function updateTemps()
{
	compTemp.refresh(p6500.sensors.compressorTemperature.toFixed(2));
	cabTemp.refresh(p6500.sensors.cabinetTemperature.toFixed(2));
}
function updateSystems()
{
	var s1 = document.getElementById('s1status');
	if(p6500.systems[0].status == "ONLINE")
	{
		s1.style.color = "#52d073"
	}
	else if(p6500.systems[0].status == "STANDBY")
	{
		s1.style.color = "#b2b8bc"
	}
	else if(p6500.systems[0].status == "SHUTDOWN")
	{
		s1.style.color = "#df7354"
	}
	s1.innerHTML = p6500.systems[0].status

	s1hum.refresh(p6500.sensors.systemOneHumidity.toFixed(2));

	var lastRuntime = document.getElementById('s1LastRuntime');
	lastRuntime.value = p6500.systems[0].compressorLastRunTimeMinutes.pad() + ":" +p6500.systems[0].compressorLastRunTimeSeconds.pad();
	var totalRuntime = document.getElementById('s1TotalRuntime');
	totalRuntime.value = p6500.systems[0].compressorTotalRuntimeHours;

	var s1 = document.getElementById('s2status');
	if(p6500.systems[1].status == "ONLINE")
	{
		s1.style.color = "#52d073"
	}
	else if(p6500.systems[1].status == "STANDBY")
	{
		s1.style.color = "#b2b8bc"
	}
	else if(p6500.systems[1].status == "SHUTDOWN")
	{
		s1.style.color = "#df7354"
	}
	s1.innerHTML = p6500.systems[1].status

	s2hum.refresh(p6500.sensors.systemTwoHumidity.toFixed(2));

	var lastRuntime = document.getElementById('s2LastRuntime');
	lastRuntime.value = p6500.systems[1].compressorLastRunTimeMinutes.pad() + ":" +p6500.systems[1].compressorLastRunTimeSeconds.pad();
	var totalRuntime = document.getElementById('s2TotalRuntime');
	totalRuntime.value = p6500.systems[1].compressorTotalRuntimeHours;

}

function updateCycleMode()
{

	var cycleMode = document.getElementById('cycleMode');

	if(p6500.mode == "BOTH")
	{
		cycleMode.selectedIndex = 0;
	}
	else if(p6500.mode == "TWENTY_FOUR_HOUR")
	{
		cycleMode.selectedIndex = 1;
	}
	else if(p6500.mode == "SYSTEM_1_ONLY")
	{
		cycleMode.selectedIndex = 2;
	}
	else if(p6500.mode == "SYSTEM_2_ONLY")
	{
		cycleMode.selectedIndex = 3;
	}

}
function withinRange(low, high, number)
{
	if(number < low)
	{
		return false;
	}
	else if (number > high)
	{
		return false;
	}
	return true;
}
function setS1Humidity()
{
	if(p6500.emulationEnabled)
	{

		var hum = prompt("Enter System 1 Humidity:")

		if(isNaN(hum))
		{
			alert("Please Enter a Number");
			return;
		}
		if(hum == null || hum == '')
		{
			return;
		}

		if(withinRange(0, 100,hum))
		{
			sendCommand(socket,"SET SYSTEM ONE HUMIDITY",hum);

		}
		else
		{
			alert("Input out of range")
		}	
	}

}
function setS2Humidity()
{
	if(p6500.emulationEnabled)
	{

		var hum = prompt("Enter System 2 Humidity:")
		if(hum == null || hum == '')
		{
			return;
		}
		if(isNaN(hum))
		{
			alert("Please Enter a Number");
			return;
		}

		if(withinRange(0, 100,hum))
		{
			sendCommand(socket,"SET SYSTEM TWO HUMIDITY",hum);

		}
		else
		{
			alert("Input out of range")
		}	
	}

}
function setTankPressure()
{
	if(p6500.emulationEnabled)
	{

		var pres = prompt("Enter Tank Pressure:")
		if(pres == null || pres == '')
		{

			return;
		}
		if(isNaN(pres))
		{
			alert("Please Enter a Number");
			return;
		}

		if(withinRange(0, 100,pres))
		{
			sendCommand(socket,"SET TANK PRESSURE",pres);

		}
		else
		{
			alert("Input out of range")
		}	
	}

}
function setCycleMode()
{
	var cycleMode = document.getElementById('cycleMode');
	var mode = cycleMode.options[cycleMode.selectedIndex].value;
	if(mode == "SYSTEM 1 ONLY")
	{
		sendCommand(socket,"SET CYCLE MODE","SYSTEM_1_ONLY");
	}
	if(mode == "BOTH")
	{
		sendCommand(socket,"SET CYCLE MODE","BOTH");
	}
	if(mode == "SYSTEM 2 ONLY")
	{
		sendCommand(socket,"SET CYCLE MODE","SYSTEM_2_ONLY");
	}
	if(mode == "24 HOUR")
	{
		sendCommand(socket,"SET CYCLE MODE","TWENTY_FOUR_HOUR");
	}	
}
function simulateCompressor()
{
	var fillRate = 2.0; //psi/sec;

	if(p6500.systems[0].compressorEnabled && !p6500.systems[1].compressorEnabled)
	{
		var pres = p6500.sensors.tankPressure + fillRate;
		sendCommand(socket,"SET TANK PRESSURE",pres);
	}
	else if(!p6500.systems[0].compressorEnabled && p6500.systems[1].compressorEnabled)
	{
		var pres = p6500.sensors.tankPressure + fillRate;
		sendCommand(socket,"SET TANK PRESSURE",pres);
	}
	else if(p6500.systems[0].compressorEnabled && p6500.systems[1].compressorEnabled)
	{
		var pres = p6500.sensors.tankPressure + 2*fillRate;
		sendCommand(socket,"SET TANK PRESSURE",pres);
	}
	else if(!p6500.systems[0].compressorEnabled && !p6500.systems[1].compressorEnabled)
	{
		var pres = p6500.sensors.tankPressure - fillRate;
		if(pres < 0)
		{
			pres = 0;
		}
		sendCommand(socket,"SET TANK PRESSURE",pres);
	}
}
function setOutletPressure()
{
	if(p6500.emulationEnabled)
	{

		var pres = prompt("Enter Outlet Pressure:")
		if(pres == null || pres == '')
		{
			return;
		}
		if(isNaN(pres))
		{
			alert("Please Enter a Number");
			return;
		}

		if(withinRange(0, 100,pres))
		{
			sendCommand(socket,"SET OUTLET PRESSURE",pres);

		}
		else
		{
			alert("Input out of range")
		}	
	}

}
function setFlow()
{
	if(p6500.emulationEnabled)
	{

		var flow = prompt("Enter Flow:")

		if(isNaN(flow))
		{
			alert("Please Enter a Number");
			return;
		}
		if(flow == null || flow == '')
		{
			return;
		}

		if(withinRange(0, 6500,flow))
		{
			sendCommand(socket,"SET FLOW",flow);

		}
		else
		{
			alert("Input out of range")
		}	
	}

}
function setCompressorTemp()
{
	if(p6500.emulationEnabled)
	{

		var temp = prompt("Enter Compresor Temperature:")

		if(isNaN(temp))
		{
			alert("Please Enter a Number");
			return;
		}
		if(temp == null || temp == '')
		{
			return;
		}

		if(withinRange(0, 200,temp))
		{
			sendCommand(socket,"SET COMPRESSOR TEMP",temp);

		}
		else
		{
			alert("Input out of range")
		}	
	}

}
function setCabinTemp()
{
	if(p6500.emulationEnabled)
	{

		var temp = prompt("Enter Cabinet Temperature:")

		if(isNaN(temp))
		{
			alert("Please Enter a Number");
			return;
		}
		if(temp == null || temp == '')
		{
			return;
		}

		if(withinRange(0, 200,temp))
		{
			sendCommand(socket,"SET CABINET TEMP",temp);

		}
		else
		{
			alert("Input out of range")
		}	
	}

}


function onResetSys1Runtime()
{
	sendMessage(socket,"RESET SYSTEM 1 RUNTIME")
}

function onResetSys2Runtime()
{
	sendMessage(socket,"RESET SYSTEM 2 RUNTIME")
}
