package com.AltecAir.CycleKit.web.server;

import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;

import com.AltecAir.CycleKit.Networking.PiNetwork;
import com.AltecAir.CycleKit.Roster.RosterController;
import com.AltecAir.CycleKit.Roster.RosterModel;
import com.AltecAir.CycleKit.Roster.WebPorts;
import com.AltecAir.CycleKit.Roster.WebStatus;
import com.AltecAir.CycleKit.Roster.RosterModel.CycleKitMode;
import com.AltecAir.CycleKit.Roster.RosterModel.PortStates;
import com.AltecAir.CycleKit.Roster.Schedule;
import com.AltecAir.CycleKit.Roster.WebAbout;
import com.AltecAir.CycleKit.Screens.SetupScreens.System.FirmwareUpdatingScreen;
import com.AltecAir.CycleKit.System.CycleKit;
import com.AltecAir.CycleKit.database.EventDatabase;
import com.AltecAir.CycleKit.states.InterfaceStateMachine;
import com.AltecAir.CycleKit.states.SystemSetup.NullFirmwareUpdateState;
import com.AltecAir.Common.org.nanohttpd.protocols.http.IHTTPSession;
import com.AltecAir.Common.org.nanohttpd.protocols.http.NanoHTTPD;
import com.AltecAir.Common.org.nanohttpd.protocols.websockets.CloseCode;
import com.AltecAir.Common.org.nanohttpd.protocols.websockets.NanoWSD;
import com.AltecAir.Common.org.nanohttpd.protocols.websockets.WebSocket;
import com.AltecAir.Common.org.nanohttpd.protocols.websockets.WebSocketFrame;
import com.AltecAir.Common.util.NetworkTools;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

public class CycleKitWebSocketServer extends NanoWSD
{
	private static final int MINUTES_IN_A_HOUR = 60;
	private CycleKit cycleKit;
	private InterfaceStateMachine stateMachine;
	private CycleKitWebSocket websocket;
	private static boolean locked;

	private static String rosterModelStatus = "0";
	private static String dryerStatus = "0";
	private static String lockedStatus = "0";
	private static String webAbout = "0";
	private static boolean verbose = false;

	public enum ERROR_TYPE
	{
		NEXT_CYCLE_TOO_FAR, MIN_DRYERS_MORE_THAN_AVAILABLE_DRYERS, NEXT_CYCLE_IN_PAST, INCORRECT_KEYWORD;
	}

	public CycleKitWebSocketServer(int port, CycleKit cycleKit, InterfaceStateMachine stateMachine)
	{
		super(port);
		locked = true;
		System.out.println("A new CycleKitWebSocketServer instance");
		this.cycleKit = cycleKit;

		this.stateMachine = stateMachine;
		try
		{

			this.makeSecure(
					NanoHTTPD.makeSSLSocketFactory("/home/pi/csa/cyclekit.jks", "?C-dc5U_ue-@7Jq_".toCharArray()),
					null);
		} catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void resetRosterModelStatus()
	{
		rosterModelStatus = "0";
	}

	public CycleKitWebSocket getWebSocket()
	{
		return websocket;
	}

	public InterfaceStateMachine getStateMachine()
	{
		return stateMachine;
	}

	public CycleKit getcycleKit()
	{
		return cycleKit;
	}

	public void update()
	{
		if (websocket != null && websocket.isOpen())
		{
			websocket.update();
		}

	}

	public void ping()
	{

		if (websocket != null && websocket.isOpen())
		{
			try
			{
				websocket.ping(new byte[]
				{ 0xA });
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private static String isLocked(boolean locked)
	{
		if (locked)
		{
			return new String("{\"locked\" : true}");
		} else
		{
			return new String("{\"locked\" : false}");
		}
	}

	private static String error(ERROR_TYPE e)
	{

		switch (e)
		{
		case NEXT_CYCLE_TOO_FAR:
			return new String("{\"error\" : \"Please choose a next cycle date less than 1 week in the future\"}");
		case NEXT_CYCLE_IN_PAST:
			return new String("{\"error\" : \"Please choose a next cycle date that is in the future\"}");
		case MIN_DRYERS_MORE_THAN_AVAILABLE_DRYERS:
		{
			return new String("{\"error\" : \"Please bring more dryers online to meet selected minimum\"}");
		}
		case INCORRECT_KEYWORD:
		{
			return new String("{\"error\" : \"Incorrect Keyword!\"}");
		}
		default:
			return new String("{\"error\" : DEFAULT ERROR CASE}");

		}
	}

	@Override
	protected WebSocket openWebSocket(IHTTPSession handshake)
	{
		// TODO Auto-generated method stub
		this.websocket = new CycleKitWebSocket(this, handshake);
		return websocket;
	}

	public static class CycleKitWebSocket extends WebSocket
	{
		private final CycleKitWebSocketServer server;
		private ObjectMapper mapper = new ObjectMapper().registerModule(new JavaTimeModule());
		private Queue<WebSocketFrame> webMessageQueue;

		public CycleKitWebSocket(CycleKitWebSocketServer server, IHTTPSession handshakeRequest)
		{
			super(handshakeRequest);
			webMessageQueue = new LinkedList<WebSocketFrame>();
			mapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
			this.server = server;

			rosterModelStatus = "0";
			dryerStatus = "0";
			webAbout = "0";

		}

		@Override
		protected void onOpen()
		{
			// TODO Auto-generated method stub
			rosterModelStatus = "0";
			dryerStatus = "0";
			webAbout = "0";

			try
			{
				send(isLocked(locked));
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		@Override
		protected void onClose(CloseCode code, String reason, boolean initiatedByRemote)
		{
			// TODO Auto-generated method stub
			rosterModelStatus = "0";
			dryerStatus = "0";
			webAbout = "0";
		}

		private String getCommand(String message)
		{
			if (message.contains(","))
			{
				return message.substring(0, message.indexOf(','));
			} else
			{
				return message;
			}

		}

		private static String getData(int index, String message)
		{
			String data = message;

			// strip the command
			data = data.substring(message.indexOf(',') + 1);
			// get rid of all of the data previous to our requested data
			for (int i = 0; i < index; i++)
			{
				data = data.substring(data.indexOf(',') + 1);

			}

			if (data.contains(","))
			{
				data = data.substring(0, data.indexOf(','));
			}

			return data;
		}

		@Override
		protected void onMessage(WebSocketFrame message)
		{
			String payload = message.getTextPayload();
			switch (getCommand(payload))
			{
			case ("UNLOCK"):
			{
				if(verbose)
				{
					System.out.println("Unlock Attempt");
				}

				webMessageQueue.add(message);
				nextMessage();

				break;
			}
			case ("LOCK"):
			{
				webMessageQueue.add(message);
				nextMessage();
				break;
			}

			case ("LOCK STATUS"):
			{
				webMessageQueue.add(message);
				nextMessage();
				break;
			}

			case ("CHANGE KEYWORD"):
			{
				if(verbose)
				{
					System.out.println("Change Keyword Attempt");
				}
				webMessageQueue.add(message);
				nextMessage();
				break;
			}
			default:
			{
				if (!locked)
				{
					if(verbose)
					{
						System.out.println("System Unlocked, Adding " + message.getTextPayload() + " to Queue");
					}
					webMessageQueue.add(message);
				} else
				{
					if(verbose)
					{
						System.out.println("System Locked, throwing out " + message.getTextPayload());
					}
				}
			}
				break;
			}

		}

		public Queue<WebSocketFrame> getMessageQueue()
		{
			return webMessageQueue;
		}

		public void nextMessage()
		{
			WebSocketFrame mess = webMessageQueue.poll();
			String payload = mess.getTextPayload();
			switch (getCommand(payload))
			{
			case ("CLEAR EVENT LOG"):
			{
				server.getcycleKit().database().clearTable();
				break;
			}
			case ("SET DATE"):
			{
				// capture the time before change so that we can log it
				LocalDateTime initialDate = LocalDateTime.now();

				String data = getData(0, payload);

				// split the time and the date. There will be a space between them
				String[] dateTimeString = data.split(" ");

				// split the date into an array of [month,day,year]
				String[] splitDate = dateTimeString[0].split("/");
				// grab the month, day and year from the array
				int month = Integer.parseInt(splitDate[0]);
				int day = Integer.parseInt(splitDate[1]);
				int year = Integer.parseInt(splitDate[2]);

				// build the date string
				StringBuilder sb = new StringBuilder();
				sb.append(String.format("%02d", month) + "/");
				sb.append(String.format("%02d", day) + "/");
				sb.append(String.format("%4d", year));

				DateTimeFormatter fmt = DateTimeFormatter.ofPattern("MM/dd/yyyy");
				DateTimeFormatter databaseFormat = DateTimeFormatter.ofPattern("MM/dd/yyyy HH:mm");
				LocalDate ld = LocalDate.parse(sb.toString(), fmt);
				LocalDateTime date = LocalDateTime.of(ld, LocalDateTime.now().toLocalTime());
				if(verbose)
				{
					System.out.println(dateTimeString[1]);
				}
				String splitTimeString[] = dateTimeString[1].split(":");
				int hour = Integer.parseInt(splitTimeString[0]);
				int minute = Integer.parseInt(splitTimeString[1]);

				date = date.withHour(hour);
				date = date.withMinute(minute);
				DateTimeFormatter dateFmt = DateTimeFormatter.ofPattern("yyyyMMdd");
				DateTimeFormatter timeFmt = DateTimeFormatter.ofPattern("HH:mm:ss");
				String dateString = dateFmt.format(date);
				String timeString = timeFmt.format(date);
				EventDatabase.insertSystemData(EventDatabase.EVENTYPE_SYSTEM_SETUP, "Date Changed From \""
						+ databaseFormat.format(initialDate) + "\" to \"" + databaseFormat.format(date) + "\"");
				try
				{
					Runtime.getRuntime().exec("sudo date --set +%Y%m%d -s " + dateString);
					Thread.sleep(100);
					Runtime.getRuntime().exec("sudo date --set +%T -s " + timeString);
					Thread.sleep(100);
					// write data to RTC
					Runtime.getRuntime().exec("sudo hwclock -w");
				} catch (IOException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InterruptedException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				RosterModel.calculateNextSwitchTime(LocalDateTime.now());
				break;
			}
			case ("SET NEXT CYCLE"):
			{
				if(verbose)
				{
					System.out.println("SET NEXT CYCLE");
				}
				// capture the time before change so that we can log it
				LocalDateTime initialDate = RosterModel.getNextSwitchTime();

				String data = getData(0, payload);

				// split the time and the date. There will be a space between them
				String[] dateTimeString = data.split(" ");

				// split the date into an array of [month,day,year]
				String[] splitDate = dateTimeString[0].split("/");
				// grab the month, day and year from the array
				int month = Integer.parseInt(splitDate[0]);
				int day = Integer.parseInt(splitDate[1]);
				int year = Integer.parseInt(splitDate[2]);

				if(verbose)
				{
					System.out.println(dateTimeString[1]);
				}
				String splitTimeString[] = dateTimeString[1].split(":");
				int hour = Integer.parseInt(splitTimeString[0]);
				int minute = Integer.parseInt(splitTimeString[1]);

				// build the date string
				StringBuilder sb = new StringBuilder();
				sb.append(String.format("%02d", month) + "/");
				sb.append(String.format("%02d", day) + "/");
				sb.append(String.format("%4d", year));

				DateTimeFormatter fmt = DateTimeFormatter.ofPattern("MM/dd/yyyy");
				LocalDate ld = LocalDate.parse(sb.toString(), fmt);
				LocalDateTime date = LocalDateTime.of(ld, LocalDateTime.now().toLocalTime());
				date = date.withHour(hour);
				date = date.withMinute(minute);
				if(verbose)
				{
					System.out.println("Setting Next switch time to " + date);
				}
				if (date.isAfter(LocalDateTime.now().plusDays(7)))
				{
					try
					{
						send(error(ERROR_TYPE.NEXT_CYCLE_TOO_FAR));
					} catch (IOException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} else if (date.isBefore(LocalDateTime.now()))
				{
					try
					{
						send(error(ERROR_TYPE.NEXT_CYCLE_IN_PAST));
					} catch (IOException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} else
				{
					RosterModel.setNextSwitchTime(date);
					RosterController.writeNextSwitchTime();
				}

				break;
			}
			case ("SET ALARM DELAY"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println("SET ALARM DELAY: " + data);
				}
				RosterModel.setAlarmDelay(Integer.parseInt(data));
				break;
			}
			case ("SET CHANGEOVER DELAY"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println("SET CHANGEOVER DELAY: " + data);
				}
				RosterModel.setChangeoverDesiredMinutes((Integer.parseInt(data)));
				break;
			}
			case ("SET MIN DRYERS"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println("SET MIN DRYERS: " + data);
				}
				int minDryers = Integer.parseInt(data);
				int availableDryers = RosterModel.getDesiredOnlineDryersInRoster();
				if(verbose)
				{
					System.out.println("Available Dryers " + availableDryers);
				}
				if (minDryers > availableDryers)
				{
					try
					{
						send(error(ERROR_TYPE.MIN_DRYERS_MORE_THAN_AVAILABLE_DRYERS));
						server.resetRosterModelStatus();
					} catch (IOException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} else
				{
					RosterModel.setMinimumDryersOnline(minDryers);
				}
				break;
			}
			case ("UNLOCK"):
			{
				String keywordAttempt = getData(0, payload);
				if(verbose)
				{
					System.out.println("Keyword Attempt: " + keywordAttempt);
				}
				String keyword = server.getcycleKit().getKeywordString();
				if(verbose)
				{
					System.out.println("Keyword: " + keyword);
				}
				String masterKeyword = server.getcycleKit().getMasterKeywordString();
				if(verbose)
				{
					System.out.println("Master Keyword: " + masterKeyword);
				}
				if (keywordAttempt.equals(keyword) || keywordAttempt.equals(masterKeyword))
				{
					locked = false;
					// server.resetRosterModelStatus();

				} else
				{
					locked = true;

					try
					{
						send(error(ERROR_TYPE.INCORRECT_KEYWORD));
					} catch (IOException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}

					server.resetRosterModelStatus();

				}

				try
				{
					send(isLocked(locked));
				} catch (IOException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				break;
			}
			case ("CHANGE KEYWORD"):
			{
				String currentKW = getData(0, payload);
				if(verbose)
				{
					System.out.println("Current Keyword: " + currentKW);
				}
				String newKW = getData(1, payload);
				if(verbose)
				{
					System.out.println("New Keyword: " + newKW);
				}
				String keyword = server.getcycleKit().getKeywordString();
				String masterKeyword = server.getcycleKit().getMasterKeywordString();
				if (currentKW.equals(keyword) || currentKW.equals(masterKeyword))
				{
					server.getcycleKit().setKeyword(Integer.parseInt(newKW));
				} else
				{
					try
					{
						send(error(ERROR_TYPE.INCORRECT_KEYWORD));
					} catch (IOException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

				break;
			}
			case ("LOCK"):
			{
				locked = true;
				break;
			}

			case ("LOCK STATUS"):
			{
				if(verbose)
				{
					System.out.println("LOCK STATUS: " + locked);
				}
				try
				{
					send(isLocked(locked));
				} catch (IOException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				break;
			}
			case ("SET MODEL NUMBER"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println("SET MODEL NUMBER: " + data);
				}
				CycleKit.setModelNumber(data);
				break;
			}
			case ("SET CYCLE MODE"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println(data);
				}

				switch (data)
				{
				case "24 HR":
					RosterModel.setCycleKitMode(CycleKitMode.TWENTY_FOUR_HOUR);
					break;
				case "48 HR":
					RosterModel.setCycleKitMode(CycleKitMode.FORTY_EIGHT_HOUR);
					break;
				case "CUSTOM":
					RosterModel.setCycleKitMode(CycleKitMode.CUSTOM);
					break;
				default:
					break;
				}
				RosterModel.calculateNextSwitchTime(LocalDateTime.now());
				break;
			}
			case ("SET CYCLE DAYS"):
			{
				boolean[] days = new boolean[7];
				for (int i = 0; i < 7; i++)
				{
					String data = getData(i, payload);
					days[i] = Boolean.parseBoolean(data);
				}
				Schedule.setDaysToSwitch(days);
				RosterModel.calculateNextSwitchTime(LocalDateTime.now());
				break;
			}
			case ("SET SERIAL NUMBER"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println("SET MODEL NUMBER: " + data);
				}
				CycleKit.setSerialNumber(data);
				break;
			}
			case ("SET MASTER PASSWORD"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println("SET MASTER PASSWORD: " + data);
				}
				CycleKit.setMasterKeyword(Integer.parseInt(data));
				break;
			}
			case ("SET SKIP WEEKEND"):
			{

				String data = getData(0, payload);
				System.out.println(data);
				if (data.equals("true"))
				{
					if(verbose)
					{
						System.out.println("Setting Skip weekends to true");
					}
					RosterModel.setSkipWeekends(true);

				} else
				{
					if(verbose)
					{
						System.out.println("Setting Skip weekends to false");
					}
					RosterModel.setSkipWeekends(false);
				}
				RosterModel.calculateNextSwitchTime(LocalDateTime.now());
				break;
			}
			case ("SET LCD DIM"):
			{

				String data = getData(0, payload);
				System.out.println(data);
				if (data.equals("true"))
				{
					if(verbose)
					{
						System.out.println("Setting LCD dim to true");
					}
					CycleKit.setDimOn(true);

				} else
				{
					if(verbose)
					{
						System.out.println("Setting LCD dim to false");
					}
					CycleKit.setDimOn(false);
				}
				break;
			}
			case ("SET ALARM RESET"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println(data);
				}
				RosterController.clearAlarm(RosterModel.getPort(Integer.parseInt(data)));
				break;
			}
			case ("SET DESIRED ROSTER"):
			{
				if(verbose)
				{
					System.out.println("SET DESIRED ROSTER");
				}
				PortStates rosterState;
				int port = Integer.parseInt(getData(0, payload));
				String state = getData(1, payload);
				int shift = Integer.parseInt(getData(2, payload));
				if(verbose)
				{
					System.out.println("SHIFT HOURS: " + shift);
				}

				switch (state)
				{
				case "ONLINE":
				{
					rosterState = PortStates.ONLINE;
					break;
				}
				case "STANDBY":
				{
					rosterState = PortStates.STANDBY;
					break;
				}
				case "EMERGENCY":
				{
					rosterState = PortStates.EMERGENCY;
					break;
				}
				case "EXERCISE":
				{
					rosterState = PortStates.EXERCISE;

					RosterModel.setExerciseShiftHour(port, shift);
					RosterModel.setExerciseTimerMinute(port, (shift * MINUTES_IN_A_HOUR));

					RosterController.writeExerciseShiftHour();
					RosterController.writeExerciseTimerMinute();

					RosterModel.stopExerciseTimer(port);
					RosterModel.startExerciseTimer(port);

					RosterModel.updateExerciseTimer(port); // REMOVE LATER AFTER UPDATE THREAD

					break;
				}
				case "MAINTENANCE":
				{
					rosterState = PortStates.MAINTENANCE;
					RosterModel.stopExerciseTimer(port);
					break;
				}
				case "DISCONNECTED":
				{
					rosterState = PortStates.DISCONNECTED;
					break;
				}
				default:
					rosterState = PortStates.DISCONNECTED;
					break;
				}
				if (rosterState.equals(PortStates.MAINTENANCE) || rosterState.equals(PortStates.EXERCISE))
				{
					RosterModel.setDesiredPortState(RosterModel.getPort(port), rosterState);
				} else
				{
					RosterModel.stopExerciseTimer(port);
					RosterModel.setDesiredPortState(RosterModel.getPort(port), rosterState);
					RosterModel.setPreviousDesiredPortState(RosterModel.getPort(port), rosterState);
					RosterController.updateRosterState(port);
				}
				RosterModel.setExerciseShiftHour(port, 0);
				RosterModel.setExerciseTimerMinute(port, 0);
				RosterController.writeCurrentDesiredPortState(port);
				break;
			}
			case ("SET DEVICE ID"):
			{
				String data = getData(0, payload);
				if(verbose)
				{
					System.out.println("SET DEVICE ID to: " + data);
				}
				server.getcycleKit().setDeviceID(data);
				break;
			}

			case ("SET PORT ID"):
			{
				int port = Integer.parseInt(getData(0, payload));
				String portDeviceID = getData(1, payload);
				if(verbose)
				{
					System.out.println("SET PORT ID " + port + " to: " + portDeviceID);
				}
				RosterModel.setPortDeviceID(port, portDeviceID);
				break;
			}

			case ("SET IP"):
			{
				String data = getData(0, payload);

				server.getcycleKit().getNetwork().changeIPAddress(NetworkTools.stringToIP(data));
				break;
			}
			case ("SET SUBNET"):
			{
				String data = getData(0, payload);

				server.getcycleKit().getNetwork()
						.setSubnetMask((byte) (NetworkTools.convertNetmaskToCIDR(NetworkTools.stringToIP(data))));
				break;
			}
			case ("SET GATEWAY"):
			{
				String data = getData(0, payload);

				server.getcycleKit().getNetwork().setDefaultGateway(NetworkTools.stringToIP(data));
				break;
			}
			case ("SET SNMP"):
			{
				String data = getData(0, payload);

				server.getcycleKit().getNetwork().changeSNMPTrap(NetworkTools.stringToIP(data));
				break;
			}
			case ("RESET TO FACTORY DEFAULT"):
			{

				server.getcycleKit().factoryReset();
				break;
			}
			case ("FIRMWARE UPDATING"):
			{
				server.getStateMachine().performTransition(new NullFirmwareUpdateState(new FirmwareUpdatingScreen()));
				EventDatabase.insertSystemData(EventDatabase.EVENTYPE_INFO, "Firmware Updated");
				break;
			}

			}

		}

		@Override
		protected void onPong(WebSocketFrame pong)
		{

		}

		@Override
		protected void onException(IOException exception)
		{
			// TODO Auto-generated method stub

		}

		public void update()
		{
			try
			{

				String newRosterModel = mapper.writerWithDefaultPrettyPrinter().writeValueAsString(new WebStatus());
				String newPortStatus = mapper.writerWithDefaultPrettyPrinter().writeValueAsString(new WebPorts());
				String newWebAbout = mapper.writerWithDefaultPrettyPrinter().writeValueAsString(new WebAbout());
				String newLockedStatus = isLocked(locked);
				if (this.isOpen())
				{
					if (!rosterModelStatus.equals(newRosterModel))
					{
						send(newRosterModel);
						rosterModelStatus = newRosterModel;
					}

					if (!dryerStatus.equals(newPortStatus))
					{
						send(newPortStatus);
						dryerStatus = newPortStatus;
					}

					if (!webAbout.equals(newWebAbout))
					{
						send(newWebAbout);
						webAbout = newWebAbout;
					}

					if (!lockedStatus.equals(newLockedStatus))
					{
						if(verbose)
						{
							System.out.println(newLockedStatus);
						}
						send(newLockedStatus);
						lockedStatus = newLockedStatus;
					}

				}

			} catch (JsonProcessingException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
