package com.AltecAir.CycleKit.web.server;

import java.io.IOException;
import java.util.ArrayList;

import com.AltecAir.Common.org.nanohttpd.protocols.http.IHTTPSession;
import com.AltecAir.Common.org.nanohttpd.protocols.http.NanoHTTPD;
import com.AltecAir.Common.org.nanohttpd.protocols.websockets.CloseCode;
import com.AltecAir.Common.org.nanohttpd.protocols.websockets.NanoWSD;
import com.AltecAir.Common.org.nanohttpd.protocols.websockets.WebSocket;
import com.AltecAir.Common.org.nanohttpd.protocols.websockets.WebSocketFrame;
import com.AltecAir.CycleKit.System.CycleKit;
import com.AltecAir.CycleKit.database.EventDatabase;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

public class DatabaseWebSocketServer extends NanoWSD
{
	private CycleKit cycleKit;
	private DatabaseWebSocket websocket;

	public DatabaseWebSocketServer(int port, CycleKit cycleKit)
	{
		super(port);
		this.cycleKit = cycleKit;
		try
		{
			
			this.makeSecure(NanoHTTPD.makeSSLSocketFactory("/home/pi/csa/cyclekit.jks",
			        "?C-dc5U_ue-@7Jq_".toCharArray()), null);
		} catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public CycleKit getP08033W()
	{
		return cycleKit;
	}

	public void update()
	{
		if (websocket != null && websocket.isOpen())
		{
			websocket.update();
		}

	}

	public void ping()
	{

		if (websocket != null && websocket.isOpen())
		{
			try
			{
				websocket.ping(new byte[]
				{ 0xA });
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	protected WebSocket openWebSocket(IHTTPSession handshake)
	{
		// TODO Auto-generated method stub
		this.websocket = new DatabaseWebSocket(this, handshake);
		return websocket;
	}

	private static class DatabaseWebSocket extends WebSocket
	{
		private final DatabaseWebSocketServer server;
		private ObjectMapper mapper = new ObjectMapper();

		private String getCommand(String message)
		{
			if (message.contains(","))
			{
				return message.substring(0, message.indexOf(','));
			} else
			{
				return message;
			}

		}

		public DatabaseWebSocket(DatabaseWebSocketServer server, IHTTPSession handshakeRequest)
		{
			super(handshakeRequest);
			mapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
			this.server = server;

		}

		@Override
		protected void onOpen()
		{
			// TODO Auto-generated method stub

		}

		@Override
		protected void onClose(CloseCode code, String reason, boolean initiatedByRemote)
		{
			// TODO Auto-generated method stub

		}
		
		private static String getData(int index, String message)
		{
			String data = message;
			//System.out.println(data);
			// strip the command
			data = data.substring(message.indexOf(',') + 1);
			// get rid of all of the data previous to our requested data
			//System.out.println("After Stripping Command: " + data);
			for (int i = 0; i < index; i++)
			{
				data = data.substring(data.indexOf(',') + 1);
				//System.out.println("Index "+ i+ " Data: "+data);
			}
			
			if (data.contains(","))
			{
				data = data.substring(0, data.indexOf(','));
			}
			
			//System.out.println("Final Data: "+ data);
			return data;
		}
		
		@Override
		protected void onMessage(WebSocketFrame message)
		{
			String payload = message.getTextPayload();
			switch (getCommand(payload))
			{
				case ("UPDATE EVENT TABLE"):
				{

					String data = getData(0, payload);
					
					ArrayList<String> eventFilter = new  ArrayList<String>();
					ArrayList<String> dryerFilter = new  ArrayList<String>();

					boolean flag = false;
					for (int i=1; i<payload.length();i++)
					{
						data = getData(i, payload);
						if(data.equals("end"))
						{
							break;
						}
						if(data.equals("dryers"))
						{
							flag = true;
						}
						else if (flag)
						{
							dryerFilter.add(data);
						}
						else
						{
							eventFilter.add(data);
						}
					}
					
					EventDatabase.setEventFilters(eventFilter);
					EventDatabase.setDryerFilters(dryerFilter);
					
					//System.out.println("UPDATE EVENT TABLE event: "+ eventFilter.toString());
					//System.out.println("UPDATE EVENT TABLE dryer: "+ dryerFilter.toString());
					
					server.getP08033W().database().setUpdateAvailable();
					
					break;
				}
			}

		}

		@Override
		protected void onPong(WebSocketFrame pong)
		{

		}

		@Override
		protected void onException(IOException exception)
		{
			// TODO Auto-generated method stub

		}

		public void update()
		{
			try
			{
				String json = EventDatabase.getFilteredTable();
				send(json);
			} catch (JsonProcessingException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
